// fldriter.cpp
//
// Source file for a palette folder iterator
// object
//


#include "fldriter.h"

// This is the static class member that is the path to the palettes folder
char PaletteFolderIterator::palettesFolderPath[_MAX_PATH];


PaletteFolderIterator::PaletteFolderIterator()
{
	curDepth = 0;
	filePathName[0] = '\0';
	searchStack = NULL;
}

PaletteFolderIterator::~PaletteFolderIterator()
{
	if(searchStack) {
		free(searchStack);
	}
}


// --- InitFromParentDir
//
// This routine takes a path string that is
// the path of the palette folder and tries
// to initialize the iterator to loop 
// over all the files in the folder.
//
// It returns true if initialization was
// successful and false otherwise
BOOL PaletteFolderIterator::InitPaletteFolderIterator()
{
	BOOL retVal = TRUE;
	
	if(SetPalettesFolderPath()) {
		// if a search stack was already allocated before
		// be sure to free the old one before creating
		// a new one
		if(NULL != searchStack) {
			free(searchStack);
			searchStack = NULL;
		}

		// try to alloate space for the new search stack
		searchStack = (PSearchStackRec) malloc(sizeof(SearchStackRec) * MAX_SEARCH_DEPTH);
		if(NULL == searchStack) {
			retVal = FALSE;
		}

		// initialize the stack table entries
		for(short ctr = 0; ctr < MAX_SEARCH_DEPTH; ctr++) {
			searchStack[ctr].searchHandle = NULL;
			searchStack[ctr].searchPath[0] = '\0';
		}

		// fill in the initial search string
		
		// create a wildcard search path from the directory
		_makepath((char *)searchStack[0].searchPath, NULL, palettesFolderPath,"*","*");
		curDepth = 0;
	} else {
		retVal = FALSE;
	}

	return retVal;
}

// --- GetNext
//
// Get the next file in line traversing into directories
// when necessary
//
// returns a pointer to a string if a file is found,
// otherwise it returns NULL
//
const char *PaletteFolderIterator::GetNext()
{
	BOOL foundFile;
	const char *retVal = NULL;

	// if curDepth is out of range, chances are that
	// something went screwy (like the user calling
	// past the end of the iteration)
	if(curDepth < 0 || curDepth > MAX_SEARCH_DEPTH - 1) 
		return NULL;

	// If the directory at the current depth
	// hasn't been started then we start a new
	// search of that directory (using searchPath)
	// to create the handle
	//
	// If a handle already exists then we can continue
	// a search we've already begun
	if(NULL == searchStack[curDepth].searchHandle) {
		HANDLE aSearchHandle;

		// Begin searching the directory
		aSearchHandle = FindFirstFile(searchStack[curDepth].searchPath,&fileData);

		// If we couldn't even start a search then set foundFile
		// to false.
		if( INVALID_HANDLE_VALUE == aSearchHandle) {
			foundFile = FALSE;
		} else {
			// if we could start the search then remember where
			// we are and continue
			searchStack[curDepth].searchHandle = aSearchHandle;
			foundFile = TRUE;
		}
	} else {
		foundFile = FindNextFile(searchStack[curDepth].searchHandle,&fileData);
	}

	if(foundFile) {
		char drive[_MAX_DRIVE];
		char dir[_MAX_DIR];
		char fname[_MAX_FNAME];
		char ext[_MAX_EXT];

		_splitpath((const char *)searchStack[curDepth].searchPath,drive,dir,fname,ext);

		if(0 == (fileData.dwFileAttributes & FILE_ATTRIBUTE_DIRECTORY)) {
			// the file we found is a true file (not a directory).

			// make a path to the file
			_makepath((char *)filePathName,drive,dir,fileData.cFileName,NULL);

			// and return it
			retVal = (char *)filePathName;
		} else {
			// the file we found was a directory
			// If we're looking at the current directory
			// or the previous directory placeholders then
			// just continue.
			if(strcmp(fileData.cFileName,".") == 0 ||
				strcmp(fileData.cFileName,"..") == 0) {
				retVal = GetNext();
			} else {
				// If we can go farther down in the search path then do so
				// otherwise just skip the directory
				if(curDepth + 1 < MAX_SEARCH_DEPTH) {
					// create a search path for the directory
					_makepath(searchStack[curDepth + 1].searchPath,drive,dir,fileData.cFileName,NULL);

					// look at all files in the directory
					_makepath(searchStack[curDepth + 1].searchPath,NULL,
								searchStack[curDepth + 1].searchPath,"*","*");

					// haven't started searching this directory
					searchStack[curDepth + 1].searchHandle = NULL;

					// push on the stack
					curDepth += 1;

					retVal = GetNext();
				} else {
					// can't go down farther so just go to next file
					retVal = GetNext();
				}
			}
		}
	} else {
		// if the stack shows we were in a directory
		if(NULL != searchStack[curDepth].searchHandle) {
			// We're done searching this directory
			FindClose(searchStack[curDepth].searchHandle);

			// Be sure old value is wiped off the stack
			searchStack[curDepth].searchHandle = NULL;

			// Pop the stack
			curDepth -= 1;

			// and return the next file in line
			if(curDepth < 0) {
				// if curDepth is less than zero then we've finished
				// searching the root palette directory
				retVal = NULL;
			} else {
				// otherwise continue with the same directory we were in
				retVal = GetNext();
			}
		} else {
			// No files were found and we're at the end of our rope
			retVal = NULL;
		}
	}

	return retVal;
}

const char *PaletteFolderIterator::GetPalettesFolderPath()
{
	if(!SetPalettesFolderPath())
		return NULL;
	else
		return palettesFolderPath;
}

// --- SetPalettesFolderPath
//
// This private routine searches the Windows registry to
// retrieve the path to the palettes folder 
//
BOOL PaletteFolderIterator::SetPalettesFolderPath()
{
	BOOL	retVal = FALSE;
	HKEY	hkMOAKeys;
	LRESULT lResult;

	palettesFolderPath[0] = '\0';

	// start by getting all the Moa Key information from the registry
	lResult = RegOpenKeyEx(HKEY_LOCAL_MACHINE,"Software\\Macromedia\\MOA",
							NULL,KEY_READ,&hkMOAKeys);

	if(ERROR_SUCCESS == lResult) {
		DWORD	dwType;
		DWORD	cbData = _MAX_PATH;

		// with the MOA registry information in hand, get the 
		// path name of the shared xtras folder.
		lResult = RegQueryValueEx(hkMOAKeys,"SharedXtrasDir",NULL,&dwType,(LPBYTE)palettesFolderPath,&cbData);
		if(ERROR_SUCCESS == lResult) {
				char drive[_MAX_DRIVE];
				char dir[_MAX_DIR];
				char fname[_MAX_FNAME];
				char ext[_MAX_EXT];

				// at this point we should have the path name to the Xtras folder.
				// in dirPath we should be able to back out one level and get the 
				// common directory then look for the "Palettes" folder.

				// we back out one level by calling split path
				_splitpath((const char *) palettesFolderPath, drive, dir, fname, ext);

				// and now we construct a new path to the Palettes folder
				_makepath((char *)palettesFolderPath,drive, dir,"Palettes", NULL);

				retVal = TRUE;
		}
		
		RegCloseKey(hkMOAKeys);
	}

	return retVal;
}